/**
  *  \file t_bitops.cc
  */

#include "cpluslib/bitops.h"

#include "tests.h"

void
TestBitOps::testShifts()
{
    TS_ASSERT_EQUALS(rol8(1, 1), 2);
    TS_ASSERT_EQUALS(rol8(1, 2), 4);
    TS_ASSERT_EQUALS(rol8(1, 4), 16);
    TS_ASSERT_EQUALS(rol8(1, 8), 1);

    TS_ASSERT_EQUALS(ror8(1, 1), 128);
    TS_ASSERT_EQUALS(ror8(1, 2), 64);
    TS_ASSERT_EQUALS(ror8(1, 4), 16);
    TS_ASSERT_EQUALS(ror8(1, 8), 1);

    TS_ASSERT_EQUALS(rol16(1, 1), 2);
    TS_ASSERT_EQUALS(rol16(1, 2), 4);
    TS_ASSERT_EQUALS(rol16(1, 4), 16);
    TS_ASSERT_EQUALS(rol16(1, 15), 32768U);
    TS_ASSERT_EQUALS(rol16(1, 16), 1);

    TS_ASSERT_EQUALS(ror16(1, 1), 32768U);
    TS_ASSERT_EQUALS(ror16(1, 2), 16384U);
    TS_ASSERT_EQUALS(ror16(1, 4), 4096U);
    TS_ASSERT_EQUALS(ror16(1, 15), 2);
    TS_ASSERT_EQUALS(ror16(1, 16), 1);

    TS_ASSERT_EQUALS(rol32(1, 1), 2);
    TS_ASSERT_EQUALS(rol32(1, 16), 65536);
    TS_ASSERT_EQUALS(rol32(0x12345678, 4), 0x23456781);
    TS_ASSERT_EQUALS(rol32(0xFFFFFFFF, 7), 0xFFFFFFFF);
    TS_ASSERT_EQUALS(rol32(1, 32), 1);
    
    TS_ASSERT_EQUALS(ror32(1, 1), 0x80000000U);
    TS_ASSERT_EQUALS(ror32(1, 16), 65536);
    TS_ASSERT_EQUALS(ror32(0x12345678, 4), 0x81234567);
    TS_ASSERT_EQUALS(ror32(0xFFFFFFFF, 7), 0xFFFFFFFF);
    TS_ASSERT_EQUALS(ror32(1, 32), 1);
}

void
TestBitOps::testPower()
{
    TS_ASSERT( isPowerOfTwo(1));
    TS_ASSERT( isPowerOfTwo(2));
    TS_ASSERT(!isPowerOfTwo(3));
    TS_ASSERT( isPowerOfTwo(4));
    TS_ASSERT(!isPowerOfTwo(5));
    TS_ASSERT(!isPowerOfTwo(6));
    TS_ASSERT(!isPowerOfTwo(7));
    TS_ASSERT( isPowerOfTwo(8));
    TS_ASSERT(!isPowerOfTwo(9));
    TS_ASSERT(!isPowerOfTwo(10));
    TS_ASSERT(!isPowerOfTwo(11));
    TS_ASSERT(!isPowerOfTwo(12));
    TS_ASSERT(!isPowerOfTwo(13));
    TS_ASSERT(!isPowerOfTwo(14));
    TS_ASSERT(!isPowerOfTwo(15));
    TS_ASSERT( isPowerOfTwo(16));
    TS_ASSERT(!isPowerOfTwo(17));
    TS_ASSERT(!isPowerOfTwo(18));
    TS_ASSERT(!isPowerOfTwo(19));

    TS_ASSERT(!isPowerOfTwo(159));
    TS_ASSERT(!isPowerOfTwo(160));
    TS_ASSERT(!isPowerOfTwo(161));

    TS_ASSERT(!isPowerOfTwo(169));
    TS_ASSERT(!isPowerOfTwo(170));
    TS_ASSERT(!isPowerOfTwo(171));

    TS_ASSERT(!isPowerOfTwo(255));
    TS_ASSERT( isPowerOfTwo(256));
    TS_ASSERT(!isPowerOfTwo(257));
}

void
TestBitOps::testFlip()
{
    TS_ASSERT_EQUALS(flipBits8(1),  0x80);
    TS_ASSERT_EQUALS(flipBits16(1), 0x8000);
    TS_ASSERT_EQUALS(flipBits32(1), 0x80000000U);

    TS_ASSERT_EQUALS(flipBits8(0x12),        0x48);
    TS_ASSERT_EQUALS(flipBits16(0x1234),     0x2C48);
    TS_ASSERT_EQUALS(flipBits32(0x12345678), 0x1E6A2C48U);

    TS_ASSERT_EQUALS(flipBits8(0),  0x00);
    TS_ASSERT_EQUALS(flipBits8(1),  0x80);
    TS_ASSERT_EQUALS(flipBits8(2),  0x40);
    TS_ASSERT_EQUALS(flipBits8(3),  0xC0);
    TS_ASSERT_EQUALS(flipBits8(4),  0x20);
    TS_ASSERT_EQUALS(flipBits8(5),  0xA0);
    TS_ASSERT_EQUALS(flipBits8(6),  0x60);
    TS_ASSERT_EQUALS(flipBits8(7),  0xE0);
    TS_ASSERT_EQUALS(flipBits8(8),  0x10);
    TS_ASSERT_EQUALS(flipBits8(9),  0x90);
    TS_ASSERT_EQUALS(flipBits8(10), 0x50);
    TS_ASSERT_EQUALS(flipBits8(11), 0xD0);
    TS_ASSERT_EQUALS(flipBits8(12), 0x30);
    TS_ASSERT_EQUALS(flipBits8(13), 0xB0);
    TS_ASSERT_EQUALS(flipBits8(14), 0x70);
    TS_ASSERT_EQUALS(flipBits8(15), 0xF0);
}
